<?php
/*
 * © Copyright 2012 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Edit participants action for a training
 * @package iHRIS
 * @subpackage Common
 * @access public
 * @author  Carl Leitner <litlfred@ibiblio.org> 
 * @copyright Copyright &copy; 2012 IntraHealth International, Inc. 
 * @since v4.1.3
 * @version v4.1.3
 */

/**
 * The action page class for editing particpants for a training
 * @package iHRIS
 * @subpackage Train
 * @access public
 */
class I2CE_PageGenerateRelationshipTemplate extends I2CE_PageActionRelationship {
    /**
     * The charset for this display
     * @var $charset
     */
    protected $charset = 'UTF-8';

    /**
     * Should html encoding be done on the data?
     * @var $encoding
     */
    protected $encoding = true;

    protected $template_file = false;
    protected $template_vars = array();
    protected $template_var_options = array();
    protected $template_var_keys = array();
    /**
     * Perform the main actions of the page.
     * @return boolean
     */
    protected function action() {
        if ( !parent::action() ) {
            I2CE::raiseError("Base action failed");
            return false;
        }
        $this->getODTTemplate();
        if (! ($this->template_file)) {
            I2CE::raiseError("No Template");
            return false;
        }        
        if (! ($this->getTemplateVariables())) {
            I2CE::raiseError("No template variables");
            return false;
        }
        if (!$this->loadData()) {
            I2CE::raiseError("could not load data");
            return false;
        }
        $this->generateTemplate();
    }

    protected function getFields() {
        $vars = $this->template_vars;
        foreach ($vars as &$fields){
            if (!in_array('id',$fields)) {
                $fields[] = 'id';
            }
        }
        unset($fields);
        return $vars;
    }


    protected function generateTemplate() {
        I2CE::longExecution( array( "max_execution_time" => 1800 ) );
        $odf_config = array( 'DELIMITER_LEFT' => '{{{',
                'DELIMITER_RIGHT' => '}}}',
                'ZIP_PROXY' => 'PhpZipProxy',
                );
        $odf = new I2CE_Odf( $this->template_file, $odf_config );
        try {
            $global_segment = $odf->setGlobalSegment();
        } catch ( OdfException $e ) {
            I2CE::raiseError( "Couldn't set global segement");
            return false;
        }
        $user = new I2CE_User();
        $name = $user->firstname . ' ' . $user->lastname;
        try {
            $odf->setVars( '++user_name', $name, $this->encoding, $this->charset );
        } catch ( OdfException $e ) {
            //It's ok if it's not there so don't do anything.
        }
        $time = strftime("%c");
        try {
            $odf->setVars( '++time', $time, $this->encoding, $this->charset );
        } catch ( OdfException $e ) {
            //It's ok if it's not there so don't do anything.
        }

        if (array_key_exists('segement_break',$this->args) && $this->args['segment_break']) {
            $global_segment->setSegmentBreak($this->args['segment_break']);
        }
        $ff = I2CE_FormFactory::instance();
        $formObjs = array();
        $row = 0;
        foreach ($this->data as $formfields) {
            $row++;
            try {
                $global_segment->setVars( '++row_num', $row, $this->encoding, $this->charset );
            } catch( SegmentException $e ) {
                //do nothing
            }
            foreach ($formfields as $reportform=>$fields) {
                $form = $this->formRelationship->getForm($reportform);
                if (!array_key_exists($reportform,$formObjs)) {
                    $formObjs[$reportform] = $ff->createContainer($form);
                }
                $formObj = $formObjs[$reportform];
                $blank_fields =array();
                foreach ($this->template_vars[$reportform] as $field) {
                    if (!array_key_exists($field,$fields)) {
                        $fields[$field] = '';
                    }
                }
                foreach ($fields as $field=>$value) {
                    $formfield = $reportform . '+' . $field;
                    if (!array_key_exists($formfield,$this->template_var_keys)) {
                        continue;
                    }                    
                    $options =  $this->template_var_options[$formfield];
                    $key = $this->template_var_keys[$formfield];
                    $fieldObj = $formObj->getField($field);
                    $this->setSegmentValue($global_segment,$fieldObj,$key,$value,$options);
                    if ($row == 1) {
                        try {                            
                            $odf->setStyleVars($key, $value, $this->encoding, $this->charset );
                        } catch( Exception $e ) {
                            //It's ok if it's not there so don't do anything.
                        }
                    }
                }
            }
            $global_segment->merge();
        }
        if ( ($errors = I2CE_Dumper::cleanlyEndOutputBuffers())) {
            I2CE::raiseError("Errors:\n" . $errors);
        }
        try {
            $odf->mergeGlobalSegment();
        } catch (OdfException $e) {
            I2CE::raiseError("Could not merge global segemnet");
            return false;
        }
        $odf->exportAsAttachedFile( basename($this->template_file) );
        exit;
    }

    protected function setSegmentValue($segment,$fieldObj,$key,$value,$options=array()) {
        if ($fieldObj instanceof I2CE_FormField_IMAGE) {
            $fieldObj->setFromDB($value);
            try {                    
                $segment->setImageFromField($key,$fieldObj,$options);
            }catch (Exception $e) {
                I2CE::raiseMessage("Cant set image $key" );
                //It's ok if it's not there so don't do anything.
            }
        } else {
            if ($fieldObj instanceof I2CE_FormField_MAP_MULT && array_key_exists('separator',$options)) {
                $value = explode(",",$value);
                foreach($value as &$v) {
                    $v = trim($v);
                }
                unset($v);
                $value = implode(str_replace('\n', "\n", $options['separator']),$value);
            }
            try {                    
                $segment->setVars( $key, $value, $this->encoding, $this->charset );
            } catch( Exception $e ) {
                //It's ok if it's not there so don't do anything.
            }
        }
    }

    protected function getTemplateVariables() {
        $template_contents = new ZipArchive();
        if ($template_contents->open($this->template_file)!==TRUE) {
            I2CE::raiseError("Could not extract odt file");
            return false;
        }
        $template_vars = array();
        for ($i=0; $i<$template_contents->numFiles;$i++) {
            $stats = $template_contents->statIndex($i);
            if (  $stats['name'] != 'content.xml' && $stats['name'] !='styles.xml') {
                continue;
            }
            $matches = array();
            //pull out all the template variables for processing.
            preg_match_all( '/{{{([0-9a-zA-Z_\-\+\,\\\=\.]+(\(.*?\))?)}}}/', $template_contents->getFromIndex($i), $matches, PREG_SET_ORDER);
            foreach ($matches as $match) {
                if (!is_array($match) || count($match) < 2 || !is_string($match[1]) || strlen($match[1]) == 0) {
                    continue;
                }
                $template_vars[] = $match[1];
            }
            $template_vars = array_unique($template_vars);
        }
        if (count($template_vars) == 0) {
            return false;
        }
        foreach ($template_vars as $key) {
            if ((!is_string($key) || strlen($key) < 1)) {
                continue;
            }
            if ($key[0] == '+') {
                continue;
            }
            list($namedform,$field,$t_extra) = array_pad(explode("+",$key,3),3,'');
            $extra = array();
            if (is_string($t_extra) && strlen($t_extra) > 0) {
                $t_extra = explode(',',$t_extra);
                foreach ($t_extra as $ex) {
                    list($ex_k,$ex_v) = array_pad(explode('=',$ex,2),2,'');
                    if (!$ex_k || !$ex_v) {
                        continue;
                    }
                    $extra[$ex_k] = $ex_v;
                }
            }             
            $namedform = trim($namedform);
            $field = trim($field);
            if (!array_key_exists($namedform,$this->template_vars)) {
                $this->template_vars[$namedform] = array();
            }
            $this->template_var_options[$namedform.'+'.$field] = $extra;
            $this->template_var_keys[$namedform.'+'.$field] = $key;
            $this->template_vars[$namedform][] = $field;
        }
        foreach ($this->formRelationship->getFormNames() as $reportform) {
            if (!array_key_exists($reportform,$this->template_vars)) {
                $this->template_vars[$reportform] = array();
            }
        }
        return true;
    }



    protected function getODTTemplate() {
        //print_r($this->args);
        if ( array_key_exists('template_upload',$this->args) && is_array($template_upload = $this->args['template_upload'])
             && array_key_exists('content',$template_upload) && $template_upload['content']
             && array_key_exists('name',$template_upload) && $template_upload['name'] ) {
            $name = $template_upload['name'];
            $pos = strrpos($name,'.');
            $ext ='';
            if ($pos !== false) {
                $ext = substr($name,$pos);
                $name = substr($name, 0,$pos);
            }
	    $this->template_file = tempnam(sys_get_temp_dir(), basename($name .'_' )) . $ext;
	    file_put_contents($this->template_file,$template_upload['content']);            
        } else  if ( array_key_exists('template',$this->args) && $template = $this->args['template']) {
            $this->template_file = I2CE::getFileSearch()->search('ODT_TEMPLATES',$template);
            if (!$this->template_file) {
                I2CE::raiseError("No template file found from $template");
                return false;
            }
        }


    }

    
}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
