<?php
/**
* © Copyright 2013 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package iHRIS
* @subpackage Common
* @author Luke Duncan <lduncan@intrahealth.org>
* @version v4.1.6
* @since v4.1.6
* @filesource 
*/ 
/** 
* Class iHRIS_PageCalendar
* 
* @access public
*/


abstract class iHRIS_PageCalendar extends I2CE_Page {

    /**
     * @var integer The month to display.
     */
    protected $month;

    /**
     * @var integer The year to display.
     */
    protected $year;

    /**
     * @var array The current date information from getdate()
     */
    protected $now;

    /**
     * @var array The date information from getdate() for the first day of the displayed month
     */
    protected $cal_display;

    /**
     * Set the month to start with if not in the GET array
     * @param integer $month
     */
    protected function setMonth( $month ) {
        if ( !is_numeric( $month ) ) {
            $month = $this->now['mon'];
        }
        if ( $month < 1 ) {
            $month = 1;
        } elseif ( $month > 12 ) {
            $month = 12;
        }
        $this->month = $month;
    }

    /**
     * Set the year to start with if not in the GET array
     * @param integer $year
     */
    protected function setYear( $year ) {
        if ( !is_numeric( $year ) ) {
            $year = $this->now['year'];
        }
        if ( $year < 0 ) {
            $year = 2000;
        }
        $this->year = $year;
    }

    /**
     * Initialize the page.
     */
    protected function initPage() {
        $this->now = getdate();

        if ( $this->get_exists( 'month' ) ) {
            $this->setMonth( $this->get( 'month' ) );
        } else {
            $this->setMonth( $this->now['mon'] );
        }
        if ( $this->get_exists( 'year' ) ) {
            $this->setYear( $this->get( 'year' ) );
        } else {
            $this->setYear( $this->now['year'] );
        }

        $this->cal_display = getdate( mktime( 0, 0, 0, $this->month, 1, $this->year ) );
        return true;
    }


    /**
     * Display the calendar on the page.
     * @return boolean
     */
    protected function action() {
        // Names from I2CE::getConfig()->modules->DatePicker->options->days->0-6
        // Names from I2CE::getConfig()->modules->DatePicker->options->months->0-11
        
        $date_opts = I2CE::getConfig()->modules->DatePicker->options;
        $this->template->addHeaderLink( "calendar.css" );
        $this->template->addFile( "calendar_base.html" );


        // Set up the date selector
        $min_year = 2000;
        if ( array_key_exists( 'min_year', $this->args ) ) {
            $min_year = $this->args['min_year'];
        } elseif ( array_key_exists( 'min_year_increment', $this->args ) ) {
            $min_year = $this->now['year'] - $this->args['min_year_increment'];
        }
        $max_year = $this->now['year'] + 5;
        if ( array_key_exists( 'max_year', $this->args ) ) {
            $max_year = $this->args['max_year'];
        } elseif ( array_key_exists( 'max_year_increment', $this->args ) ) {
            $max_year = $this->now['year'] + $this->args['max_year_increment'];
        }
        for( $year = $min_year; $year <= $max_year; $year++ ) {
            $year_select = $this->template->getElementById( 'year_select' );
            $attr = array( 'value' => $year );
            if ( $year == $this->year ) {
                $attr['selected'] = 'selected';
            }
            $opt = $this->template->createElement( 'option', $attr, $year );
            $year_select->appendChild( $opt );
        }
        for( $month = 1; $month <= 12; $month++ ) {
            $month_select = $this->template->getElementById( 'month_select' );
            $attr = array( 'value' => $month );
            if ( $month == $this->month ) {
                $attr['selected'] = 'selected';
            }
            $mon = getdate( mktime( 0, 0, 0, $month, 1, $this->year ) );
            $mon_name = $mon['month'];
            $date_opts->setIfIsSet( $mon_name, "months/" . ($month-1) );
            $opt = $this->template->createElement( 'option', $attr, $mon_name );
            $month_select->appendChild( $opt );
        }
        $this->template->setDisplayDataImmediate( 'calendar_current', 'month=' . $this->now['mon']. '&year=' . $this->now['year'] );


        // Set up the month name and weekdays
        $month_name = $this->cal_display['month'];
        $date_opts->setIfIsSet( $month_name, "months/" . ( $this->cal_display['mon'] - 1 ) );
        $this->template->setDisplayDataImmediate( 'month_name', $month_name );
        $this->template->setDisplayDataImmediate( 'year_name', $this->year );
        $prev_mon = $this->month - 1;
        $prev_year = $this->year;
        if ( $prev_mon < 1 ) {
            $prev_mon = 12;
            $prev_year--;
        }
        $this->template->setDisplayDataImmediate( 'calendar_previous_month', 'month=' . $prev_mon. '&year=' . $prev_year );
        $next_mon = $this->month + 1;
        $next_year = $this->year;
        if ( $next_mon > 12 ) {
            $next_mon = 1;
            $next_year++;
        }
        $this->template->setDisplayDataImmediate( 'calendar_next_month', 'month=' . $next_mon. '&year=' . $next_year );
        $this->template->setDisplayDataImmediate( 'calendar_previous_year', 'month=' . $this->month . '&year=' . ($this->year-1) );
        $this->template->setDisplayDataImmediate( 'calendar_next_year', 'month=' . $this->month . '&year=' . ($this->year+1) );
        for( $i = 1; $i <= 7; $i++ ) {
            $day = getdate( mktime( 0, 0, 0, $this->month, $i, $this->year ) );
            $day_name = $day['weekday'];
            $date_opts->setIfIsSet( $day_name, "days/" . $day['wday'] );
            $this->template->setDisplayDataImmediate( 'day_name_' . $day['wday'], $day_name );
        }

        $days = $this->template->getElementById( 'calendar_days' );

        $beginning = true;

        $cur_date = $this->cal_display;
        while ( $cur_date['mon'] == $this->month ) {
            if ( $beginning || $cur_date['wday'] == 0 ) {
                $row = $this->template->createElement( 'tr' );
                $days->appendChild( $row );
            }
            if ( $beginning ) {
                $beginning = false;
                if ( $cur_date['wday'] > 0 ) {
                    $blank = $this->template->appendFileByNode( 'calendar_blank.html', 'td', $row );
                    $blank->setAttribute( "colspan", $cur_date['wday'] );
                }
            }
            $mday = $cur_date['mday'];
            $day = $this->template->appendFileByNode( 'calendar_day.html', 'td', $row );
            $this->setDisplayDataImmediate( 'calendar_date', $cur_date['mday'], $day );
            $this->template->setAttribute( 'id', sprintf( "%04d_%02d_%02d", $this->year, $this->month, $mday), 
                    null, 'div[@name="calendar_text"]', $day );
            $cur_date = getdate( mktime( 0, 0, 0, $this->month, $mday+1, $this->year ) );
        }
        if ( $cur_date['wday'] > 0 ) {
            $blank = $this->template->appendFileByNode( 'calendar_blank.html', 'td', $row );
            $blank->setAttribute( "colspan", 7 - $cur_date['wday'] );
        }

        return true;

    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
